import pygraphviz as pgv

stringify = pgv.testing.stringify


def test_node_attribute():
    A = pgv.AGraph()
    A.add_node(1, label="test", spam="eggs")
    ans = """strict graph { node [label="\\N"]; 1 [label=test, spam=eggs]; }"""
    assert stringify(A) == ans


def test_node_attributes2():
    A = pgv.AGraph()
    A.add_node(1)
    one = A.get_node(1)
    one.attr["label"] = "test"
    one.attr["spam"] = "eggs"
    assert "label" in one.attr
    assert one.attr["label"] == "test"
    assert sorted(one.attr.keys()) == ["label", "spam"]
    ans = """strict graph { node [label="\\N"]; 1 [label=test, spam=eggs]; }"""
    assert stringify(A) == ans

    one.attr["label"] = ""
    one.attr["spam"] = ""
    ans = """strict graph { node [label="\\N"]; 1 [label=""]; }"""
    assert stringify(A) == ans

    one.attr["label"] = "test"
    del one.attr["label"]
    ans = """strict graph { node [label="\\N"]; 1 [label=""]; }"""
    assert stringify(A) == ans


def test_node_attribute_update():
    A = pgv.AGraph()
    A.add_node(1, label="test", spam="eggs")
    A.add_node(1, label="updated")
    ans = """strict graph { node [label="\\N"]; 1 [label=updated, spam=eggs]; }"""
    assert stringify(A) == ans


def test_node_attribute_remove():
    A = pgv.AGraph()
    A.add_node(1, label="test", spam="eggs")
    A.add_node(1, label=r"\N", spam="")  # use \N to signify null label, else ''
    ans = """strict graph { node [label="\\N"]; 1; }"""
    assert stringify(A) == ans



import os
import pytest

import pygraphviz as pgv
from pygraphviz.scraper import _get_sg_image_scraper


def test_scraper(tmpdir):
    pytest.importorskip("sphinx_gallery")
    scraper = _get_sg_image_scraper()

    ### Source
    src_dir = str(tmpdir)
    src_file = os.path.join(src_dir, "simple.py")  # no need for this to exist
    # Create source PNG
    A = pgv.AGraph()
    A.add_edge(1, 2)
    A.layout()
    A.draw(os.path.join(src_dir, "simple.png"))

    ### Target
    out_dir = os.path.join(src_dir, "build", "html")
    os.makedirs(out_dir)
    out_file = os.path.join(out_dir, "simple.png")
    # Target should **not** exist
    assert not os.path.isfile(out_file)
    # Copy source PNG to target location
    block = None
    block_vars = dict(
        image_path_iterator=(img for img in [out_file]),
        src_file=src_file,
    )
    gallery_conf = {"src_dir": src_dir, "builder_name": "html"}
    scraper(block, block_vars, gallery_conf)
    # Target should exist
    assert os.path.isfile(out_file)



import os
from glob import glob
import shutil


def _get_sg_image_scraper():
    """Return the callable scraper to be used by Sphinx-Gallery.

    It allows PyGraphviz users to just use strings as they already can for
    'matplotlib' and 'mayavi'. Details on this implementation can be found in
    `sphinx-gallery/sphinx-gallery/494`_

    .. _sphinx-gallery/sphinx-gallery/494: https://github.com/sphinx-gallery/sphinx-gallery/pull/494
    """
    return PNGScraper()


class PNGScraper:
    """A callable image scraper for png outputs from pygraphviz examples
    for sphinx-gallery.

    Can be used to collect `.png` files generated by pygraphviz in
    sphinx-gallery examples by adding the following to the ``conf.py``::

        sphinx_gallery_conf = {
            ...
            'image_scrapers': ('matplotlib', 'pygraphviz'),
        }

    This class is based on the recipe provide in [1]_.

    References
    ----------
    .. [1] sphinx-gallery documentation - custom image scraper.
       https://sphinx-gallery.github.io/stable/advanced.html
    """

    def __init__(self):
        self.seen = set()

    def __repr__(self):
        return "PNGScraper"

    def __call__(self, block, block_vars, gallery_conf):
        """
        Scrape .png images created by pygraphviz examples.

        Invoked by sphinx-gallery.

        Parameters
        ----------
        block : tuple
            A tuple containing the (label, content, line_number of the
            block.
        block_vars : dict
            Dict of block variables
        gallery_conf : dict
            Mapping of sphinx-gallery configuration values

        Returns
        -------
        str :
            rST-formatted string containing the generated images. This will
            be rendered to HTML during the ``sphinx-build`` process.
        """
        try:
            from sphinx_gallery.scrapers import figure_rst
        except ImportError as e:
            raise ImportError("You must install `sphinx_gallery`") from e

        # Find all PNG files in the directory of this example
        path_current_example = os.path.dirname(block_vars["src_file"])
        pngs = sorted(glob(os.path.join(path_current_example, "*.png")))

        # Iterate through PNGs and copy them to sphinx-gallery output dir
        image_names = []
        image_path_iterator = block_vars["image_path_iterator"]
        for png in pngs:
            if png not in self.seen:
                self.seen |= set(png)
                this_image_path = next(image_path_iterator)
                image_names.append(this_image_path)
                shutil.move(png, this_image_path)

        # Use figure_rst to generate rST for image files
        return figure_rst(image_names, gallery_conf["src_dir"])



"""
Knuth Miles
===========

An example that shows how to add your own positions to nodes
and have graphviz "neato" position the edges. 

miles_graph() returns an undirected graph over the 128 US cities from
the datafile miles_dat.txt.  

This example is described in Section 1.1 in Knuth's book [1]_ [2]_.

The data used in this example is copied from [2]_.  The filename and
header have been modified to adhere to the request of the author to
not corrupt the original source file content and name.

References.
-----------
.. [1] Donald E. Knuth,
   "The Stanford GraphBase: A Platform for Combinatorial Computing",
   ACM Press, New York, 1993.
.. [2] http://www-cs-faculty.stanford.edu/~knuth/sgb.html
"""

__author__ = """Aric Hagberg (aric.hagberg@gmail.com)"""


def miles_graph():
    """Return a graph from the data in miles_dat.txt.

    Edges are made between cities that are less then 300 miles apart.
    """
    import math
    import re
    import gzip

    G = pgv.AGraph(name="miles_dat")
    G.node_attr["shape"] = "circle"
    G.node_attr["fixedsize"] = "true"
    G.node_attr["fontsize"] = "8"
    G.node_attr["style"] = "filled"
    G.graph_attr["outputorder"] = "edgesfirst"
    G.graph_attr["label"] = "miles_dat"
    G.graph_attr["ratio"] = "1.0"
    G.edge_attr["color"] = "#1100FF"
    G.edge_attr["style"] = "setlinewidth(2)"

    cities = []
    for line in gzip.open("miles_dat.txt.gz", "rt"):
        if line.startswith("*"):  # skip comments
            continue
        numfind = re.compile(r"^\d+")

        if numfind.match(line):  # this line is distances
            dist = line.split()
            for d in dist:
                if float(d) < 300:  # connect if closer then 300 miles
                    G.add_edge(city, cities[i])
                i = i + 1
        else:  # this line is a city, position, population
            i = 1
            (city, coordpop) = line.split("[")
            cities.insert(0, city)
            (coord, pop) = coordpop.split("]")
            (y, x) = coord.split(",")
            G.add_node(city)
            n = G.get_node(city)
            # assign positions, scale to be something reasonable in points
            n.attr["pos"] = "%f,%f)" % (
                -(float(x) - 7000) / 10.0,
                (float(y) - 2000) / 10.0,
            )
            # assign node size, in sqrt of 1,000,000's of people
            d = math.sqrt(float(pop) / 1000000.0)
            n.attr["height"] = "%s" % (d / 2)
            n.attr["width"] = "%s" % (d / 2)
            # assign node color
            n.attr["fillcolor"] = "#0000%2x" % (int(d * 256))
            # empty labels
            n.attr["label"] = " "

    return G


if __name__ == "__main__":
    import warnings
    import pygraphviz as pgv

    # ignore Graphviz warning messages
    warnings.simplefilter("ignore", RuntimeWarning)

    G = miles_graph()
    print("Loaded miles_dat.txt containing 128 cities.")

    G.write("miles.dot")
    print("Wrote miles.dot")
    G.draw("miles.png", prog="neato", args="-n2")
    print("Wrote miles.png")



import pygraphviz as pgv
import pytest


def test_repr_output():
    A = pgv.AGraph()
    A.add_path([1, 2, 3, 4])
    assert repr(A).startswith("<AGraph <Swig Object of type 'Agraph_t")
    A.layout()
    assert A._svg_repr().startswith("<?xml version=")


def test_mimetype_select():
    A = pgv.AGraph()
    A.add_path([1, 2, 3, 4])
    assert "text/plain" in A._repr_mimebundle_().keys()
    assert not "image/svg+xml" in A._repr_mimebundle_().keys()
    A.layout()
    assert "image/svg+xml" in A._repr_mimebundle_().keys()


def test_svg_repr_error():
    with pytest.raises(AttributeError):
        A = pgv.AGraph()
        A._svg_repr()
